***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### Eab2HF ###

c     Evaluates off-diagonal Lagrange multipliers in cases
c     occ(iorb1)<>occ(iorb2)

c      e(iorb2,iorb1) = e(2,1) = <2|V_C-V_x|1>
c      e(iorb1,iorb2) = e(1,2) = <1|V_C-V_x|2>

      subroutine Eab2HF(iorb1,iorb2,psi,pot,excp,wgt2,wk0,wk1)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)

      include 'commons8.inc'
      dimension psi(*),pot(*),excp(*),wgt2(*),wk0(*),wk1(*)

      ipc12=iorb1+(iorb2-1)*norb
      engo(ipc12)=zero

      if (mm(iorb1).ne.mm(iorb2)) return
      if (ige(iorb1).ne.ige(iorb2)) return

      i1beg1=i1b(iorb1)
      i1beg2=i1b(iorb2)
      ngorb1=i1si(iorb1)
      ngpot1=i2si(iorb1)	

      ngorb2=i1si(iorb2)     
      ngpot2=i2si(iorb2)	

      norb2=norb*norb

c     add contributions from coulomb and exchange potentials

      do iorb=1,norb
         i1beg=i1b(iorb)
         ngorb=i1si(iorb)		
         ngpot=i2si(iorb)
         
         kex=iorb2+norb*(iorb-1)
         if (iorb2.le.iorb) then
            ihc=iorb2+iorb*(iorb-1)/2
         else
            ihc=iorb+iorb2*(iorb2-1)/2
         endif

         i2beg=i2b(iorb)
         i3beg=i3b(ihc)
         ngex =i3si(ihc)
         
         ngrid=min(ngorb2,ngpot)		
         do i=1,mxsize
            wk0(i)=0.d0
         enddo	    

         coo=occ(iorb)
         if (iorb2.eq.iorb) coo=coo-zero
         
         call copy (ngrid,pot(i2beg),ione,wk0,ione)
         call prod  (ngrid,psi(i1beg2),wk0)

         if (coo.ne.zero) then
            call scal (ngrid,coo,wk0,ione)
         endif

         ngrid2=min(ngorb2,ngex,ngorb)		
         if (iorb.ne.iorb2)  then
            coo=gec(kex)
            call prodas (ngrid2,-coo,psi(i1beg),excp(i3beg),wk0)
            if (ilc(ihc).gt.1) then
               coo=gec(kex+norb*norb)
               call prodas (ngrid2,-coo,psi(i1beg),
     &              excp(i3beg+i3si(ihc)),wk0)
            endif
         else
            if ((mm(iorb2).gt.0).and.(ilc(ihc).gt.0)) then
               coo=gec(kex)
               call prodas (ngrid2,-coo,psi(i1beg),excp(i3beg),wk0)
            endif
         endif
       
         if (iorb.eq.1) then
            call copy (ngrid,wk0,ione,wk1,ione)
         else
            call add   (ngrid,wk0,wk1)
         endif
      enddo
      
c     to complete the integrand wk1 has to be multiplied by psi(i1beg1)
      call prod (ngrid,psi(i1beg1),wk1)
      wtwoel=dot(ngrid,wgt2,ione,wk1,ione)

      engo(ipc12)= wtwoel
      
      return
      end
