***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### qvwn ###
c
c     Calculates q function of VWN functional (see
c     http://wild.life.nctu.edu.tw/~jsyu/molpro2002.1/doc/manual/node184.html)

      real*8 function qvwn(x,a,p,c,d)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'	

      xcap=x*x+c*x+d
      qcap=sqrt(four*d-c*c)      
      arctan=atan(qcap/(two*x+c))

      t1=log(x*x/xcap)
      t2=two*c/qcap*arctan
      t3=-c*p/xcap*(log((x-p)*(x-p)/xcap) +two*(c+two*p)/qcap*arctan)
      
      qvwn=a*(t1+t2+t3)
      return
      end

c ### qvwnderx ###

c     Calculates dq/dx of the VWN functional 

      real*8 function qvwnderx(x,a,p,c,d)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'	

c X=xcap, dX/dx=xcapder 
      xcap=x*x+c*x+d
      xcapder=two*x+c
      xcapder2=xcapder*xcapder

      xcapp=p*p+c*p+d

      qcap=sqrt(four*d-c*c)      
      qcap2=(four*d-c*c)      

      t1=two/x-xcapder/xcap
      t2=-four*c/(qcap2+xcapder2)
      t3=four*c*p*(c+two*p)/xcapp/(qcap2+xcapder2)
      t4=-c*p/xcapp*(two/(x-p)-xcapder/xcap)
      
      qvwnderx=a*(t1+t2+t3+t4)
      return
      end

c ### ff ###
c     Calculates F(s) (see fpw86sup)

      real*8 function ff (s)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const115=1.d0/15.d0)
      ff=ffbar(s)**const115

      return
      end

c ### ffbar ###
c     Calculates Fbar(s) (see fpw86sup)

      real*8 function ffbar (s)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (a=1.296d0,b=14.d0,c=0.2d0)
      s2=s*s
      ffbar=(1.d0+a*s2+b*s2*s2+c*s2*s2*s2)

      return
      end

c ### ffbarp ###
c     Calculates Fbar(s) prime (see fpw86sup)

      real*8 function ffbarp (s)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (a=1.296d0,b=14.d0,c=0.2d0)

      s2=s*s
      ffbarp=(2.d0*a*s+4.d0*b*s*s2+6.d0*c*s*s2*s2)

      return
      end


c ### ffp ###
c     Calculates dF(s)/ds (see fpw86sup)

      real*8 function ffp (s)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const43=4.d0/3.d0,
     &     const115=1.d0/15.d0)
      parameter (a=1.296d0,b=14.d0,c=0.2d0)

      s2=s*s
      ffp=const115*ffbar(s)**(const115-1.d0)*ffbarp(s)

      return
      end


c ### ffdp ###
c     Calculates d(1/s dF(s)/ds)/ds (see fpw86sup)

      real*8 function ffdp (s)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const43=4.d0/3.d0,
     &     const115=1.d0/15.d0)
      parameter (a=1.296d0,b=14.d0,c=0.2d0)

      s2=s*s
      ffdp= const115*
     &            (8.d0*b*s+24.d0*c*s*s2)*ffbar(s)**(const115-1.d0)
     &     +const115*(const115-1.d0)*
     &            ffbarp(s)**2.d0/s*ffbar(s)**(const115-2.d0)

      return
      end


