***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### doSCF ####

c     Controls SCF process.

      subroutine doSCF (cw_sor,cw_orb,cw_coul,cw_exch,cw_suppl,cw_sctch)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      logical stop_x2dhf
      integer*4 cw_sor
      include 'commons8.inc'
      dimension cw_orb(*),cw_coul(*),cw_exch(*),
     &		cw_suppl(*),cw_sctch(*)
      dimension demaxt(60)
      dimension iorborder(7)
      common /status/ istat

      istat=1
      stop_x2dhf=.false.
      ic=0
      thren=10.d0**dble(-ienterm)
      thrno=10.d0**dble(-inoterm)

c     the SCF procedure starts here: set some parameters and go

      ddmax=0.d0
      do i=1,norb
         itouch(i)=0
         demax(i)=100.d0
         denmax=100.d0
         demaxt(i)=0.d0
         engi(i)=eng(i)
      enddo
      dnomax=100.d0

      tscf=0.d0
      trelax=0.d0
      tortho=0.d0
      trayl =0.d0
      tlagra=0.d0	
      tmomen=0.d0
      ttoten=0.d0

      count_thrno=0
      count_max_thrno=5

c     SCF/SOR process is stopped if orbital energy differences are
c     either too large or too small

      ddmaxprev=1.e35
      dnmaxprev=1.e35
      noenergydec=0
      nonormdec=0

      call getCpuTime (time1)      
      tscf=time1
      
      iscf=0
      
      time1=0.d0
      time2=0.d0

c     inforce C_i symmetry or orbitals 
      if (ihomon.eq.2) then
	 call setci(izero,cw_orb)
      endif     
      
c     calculate total energy in the case of exchange potentials being
c     kept on disk during SCF cycles

      if (iform.eq.0.or.iform.eq.2) then
         iorb=0
         call etotalOrb (iorb,cw_orb,cw_coul,cw_exch,
     &        cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),cw_suppl(i4b(13)),
     &        cw_suppl(i4b(14)),
     &        cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),cw_sctch(i5b( 3)),
     &        cw_sctch(i5b( 4)),cw_sctch(i5b( 5)),cw_sctch(i5b( 6)),
     &        cw_sctch(i5b( 7)),cw_sctch(i5b( 8)),cw_sctch(i5b( 9)),
     &        cw_sctch(i5b(10)),cw_sctch(i5b(11)),cw_sctch(i5b(12)),
     &        cw_sctch(i5b(13)),cw_sctch(i5b(14)))
      endif

c     prepear an array needed for writing exchange potentials involving a given
c     orbital 

      call prepwexch
         
      
c     ----- begin of scf loop -----
      
      ioo=1
      imomen=0
      
      if (maxscf.le.0) write(iout6,*) '... skipping scf iterations ...'

      nthren=0
      do iscf=1,maxscf

         if (iscf.eq.1) then
            if (iprint16.eq.0) then
               write(iout6,15000)
            else
               write(iout6,25000)
            endif
         endif

c        ----- loop over orbitals (in reverse order) -----

c        relaxation begins from the innermost orbital, i.e. the last one
c        on the input data list unless iscforder specifies alternative
c        ordering
         
         ibeg=1
         iend=norb
         
c FIXME
         if (iorder(i).eq.5.and.mod(iscf,itwo).eq.0) then
            ioo=-1
         else
            ioo= 1
         endif

         iasympt=norb

         do i=ibeg,iend
            if (iscforder(i).ne.0) then
               iorb=iscforder(i)
            else
               iorb=norb-i+1
            endif
            
            itouch(iorb)=1
c            if (ifix(iorb).ne.0) goto 100

            call getCpuTime (time1)

            if (iform.eq.0.or.iform.eq.2) then
c              retrieve from disk exchange potentials involving iorb	
               call rfdexch(iorb,cw_exch)
            endif

c           determines Coulomb and exchange potentials in the asymptotic region
c           from the multipole expansion

            if (iasympt.ne.0) then
               call potAsympt(iorb,cw_coul,cw_exch)  
               iasympt=iasympt-1
            endif

c           in case of near-degenerate orbitals, i.e. when performing
c           calculations for homonuclear molecules without inforced
c           symmetry (no homo label), especially when external electric
c           field is applied, one can improve convergence by changing
c           the direction of the SOR sweeps (forward/backward sweeps for
c           even/odd SCF)

            isstart= 1
            isstop = ngrd1
            isstep = 1
            
            if (ialtsweeps.eq.1) then
               if (mod(iorb,itwo).eq.0) then
                  isstart= 1
                  isstop = ngrd1
                  isstep = 1
               else
                  isstart= ngrd1
                  isstop = 1
                  isstep =-1
               endif
            endif

c     perform relaxation of Coulomb and exchange potentials
c     (maxsorpot(iorb) sweeps of each function) and orbitals
c     (maxsororb(iorb) sweeps)

            call getCpuTime(time1)
            call doSOR (iorb,cw_sor,cw_orb,cw_coul,cw_exch,
     &           cw_suppl(i4b( 1)),cw_suppl(i4b( 2)),cw_suppl(i4b( 3)),
     &           cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),cw_suppl(i4b( 6)),  
     &           cw_suppl(i4b( 7)),cw_suppl(i4b( 8)),cw_suppl(i4b( 9)),     
     &           cw_suppl(i4b(10)),cw_suppl(i4b(11)),cw_suppl(i4b(12)),     
     &           cw_suppl(i4b(14)),
     &           cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),cw_sctch(i5b( 3)),          
     &           cw_sctch(i5b( 4)),cw_sctch(i5b( 5)),cw_sctch(i5b( 6)),
     &           cw_sctch(i5b( 7)),cw_sctch(i5b( 8)),cw_sctch(i5b( 9)),
     &           cw_sctch(i5b(10)),cw_sctch(i5b(11)),cw_sctch(i5b(12)),
     &           cw_sctch(i5b(13)),cw_sctch(i5b(14)))
            call getCpuTime (time2)
            trelax=trelax+(time2-time1)

c           normalize the current orbital
            call getCpuTime (time1)
            call norm (iorb,cw_orb,
     &           cw_suppl(i4b(9)),cw_suppl(i4b(14)),cw_sctch(i5b(1)))
            call getCpuTime (time2)

            tortho=tortho+(time2-time1)

c           enforce C_i symmetry of iorb orbital
            if (ihomon.eq.2) then
               call setci(iorb,cw_orb)
            endif

c           orthogonalize the current orbital
            call getCpuTime (time1)
            if (norb.gt.1) then
               call ortho (iorb,cw_orb,
     &              cw_suppl(i4b(9)),cw_suppl(i4b(14)),
     &              cw_sctch(i5b(1)))
            endif
            call getCpuTime(time2)
            tortho=tortho+(time2-time1)

c           call rotate(iorb,cw_orb,cw_sctch(i5b(1)))	  

c           calculate diagonal energy value
            
            call getCpuTime(time1)
            if (islat.eq.0) then            
               call Ea (iorb,cw_orb,cw_coul,cw_exch,
     &              cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &              cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &              cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &              cw_sctch(i5b( 3)),cw_sctch(i5b( 4)))
            else
               call EaDFT (iorb,cw_orb,cw_coul,cw_exch,
     &              cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &              cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &              cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &              cw_sctch(i5b( 3)),cw_sctch(i5b( 4)))
            endif
         
            call getCpuTime(time2)
            trayl =trayl + (time2-time1)	  	  

            demaxt(iorb)=eng(iorb)-engi(iorb)
c           sometimes demaxt becomes zero although it should not normally happen
c           that is why it is increased (esspecially important for OED calculations
c           when converged orbitals get 'fixed' status)
            if (abs(demaxt(iorb)).eq.0d0) demaxt(iorb)=two*thren

            engi(iorb)=eng(iorb)	  

c           calculate off-diagonal Lagrange multipliers

            call getCpuTime(time1)
            if (islat.eq.0) then            
               call Eab (iorb,cw_orb,cw_coul,cw_exch,
     &              cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &              cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &              cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &              cw_sctch(i5b( 3)),cw_sctch(i5b( 4)))
            else
               call EabDFT (iorb,cw_orb,cw_coul,cw_exch,
     &              cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &              cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &              cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &              cw_sctch(i5b( 3)),cw_sctch(i5b( 4)))
            endif
            call getCpuTime(time2)
            tlagra =tlagra + (time2-time1)	  	  

            if (iprint(50).ne.0) then
               if (islat.eq.0) then            
                  call contriborb (iorb,cw_orb,cw_coul,cw_exch,
     &                 cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &                 cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &                 cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &                 cw_sctch(i5b( 3)),cw_sctch(i5b( 4)))
               else
                  call contriborbDFT (iorb,cw_orb,cw_coul,cw_exch,
     &                 cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &                 cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &                 cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &                 cw_sctch(i5b( 3)),cw_sctch(i5b( 4)))
               endif
            endif
            
            if (iform.eq.0.or.iform.eq.1) then
               call wtdexch(iorb,cw_exch)
            endif

            if (iprtlev.eq.1) then
               if (iprint16.eq.0) then
                  write(iout6,15100) iscf,iorn(iorb),bond(iorb),
     &                 gut(iorb),eng(iorb),demaxt(iorb),1.0-area(iorb),
     &                 wstorthog(iorb)
c     &                 nodes(iorb,cw_orb)
               else
                  write(iout6,25100) iscf,iorn(iorb),bond(iorb),
     &                 gut(iorb),eng(iorb),demaxt(iorb),1.0-area(iorb),
     &                 wstorthog(iorb)
c     &                 nodes(iorb,cw_orb)
               endif
            endif     

            if (nobckup.gt.0) then
               modv=mod(iscf,abs(nobckup))
               if (modv.eq.0) then
                  if (iform.eq.0.or.iform.eq.2) then

c                    calculate 'intermidiate' total energy
                     
                     call getCpuTime(time1)
                     call etotalOrb (iorb,cw_orb,cw_coul,cw_exch,
     &                    cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &                    cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &                    cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &                    cw_sctch(i5b( 3)),cw_sctch(i5b( 4)),
     &                    cw_sctch(i5b( 5)),cw_sctch(i5b( 6)),
     &                    cw_sctch(i5b( 7)),cw_sctch(i5b( 8)),
     &                    cw_sctch(i5b( 9)),cw_sctch(i5b(10)),
     &                    cw_sctch(i5b(11)),cw_sctch(i5b(12)),
     &                    cw_sctch(i5b(13)),cw_sctch(i5b(14)))
                     call getCpuTime(time2)
                     ttoten=ttoten+(time2-time1)
                  endif
               endif
            endif

c       Sometimes a lengthy calculation needs to be stoped earlier.
c       kill <pid> will do the job but you risk that the data being
c       written will be lost. To avoid this and force the program to end
c       gracefully make  
c          touch stop_x2dhf
c       or simply
c          ./xhf stop
        
c       If this file is detected the program ends.
        
            inquire (file ='stop_x2dhf',exist=stop_x2dhf)
            if (stop_x2dhf) goto 110

c00100       continue
         enddo

c     ----- end of orbital loop -----	


         if (idbg(77).ne.0)  then
c           assign maxsor for each orbital every iepoch iterations
            call schedSOR
         endif


00110    continue
         inde=1
         denmax=0.d0

         indn=1
         dnomax=0.d0
         
         if (norb.gt.0) then
            do nei=1,norb
               if(abs(demaxt(nei)).ge.abs(denmax)) then
                  denmax=demaxt(nei)
                  inde=nei
               endif
            enddo
            do nei=1,norb
               if (abs(1.d0-area(nei)).ge.dnomax) then
                  dnomax=abs(1.d0-area(nei))
                  if (dnomax.eq.0d0) dnomax=precis
                  indn=nei
               endif
            enddo
         endif	
         
c        inde and indn point to the worst converged orbital as far as
c        orbital energies and norms are concerned, respectively
      
         if (ddmax.eq.0.d0) ddmax=denmax
         
         if (iprtlev.eq.2) then 
            if (iprint16.eq.0) then
               write(iout6,15110) iscf,iorn(inde),bond(inde),  
     &             gut(inde),eng(inde),denmax,1.0-area(inde)
               write(iout6,15110) iscf,iorn(indn),bond(indn),  
     &             gut(indn),eng(indn),demaxt(indn),1.0-area(indn)

               else
                  write(iout6,25100) iscf,iorn(inde),bond(inde),  
     &             gut(inde),eng(inde),denmax,1.0-area(inde)
               write(iout6,25100) iscf,iorn(indn),bond(indn),  
     &             gut(indn),eng(indn),demaxt(indn),1.0-area(indn)
               endif
                  
         endif


         modv=mod(iscf,abs(nobckup))
         if (modv.eq.0) then
            if (iprtlev.eq.3) then
               if (iprint16.eq.0) then
                  write(iout6,15110) iscf,iorn(inde),bond(inde),  
     &                 gut(inde),eng(inde),denmax,1.0-area(inde)
                  write(iout6,15110) iscf,iorn(indn),bond(indn),  
     &                 gut(indn),eng(indn),demaxt(indn),1.0-area(indn)
               else
                  write(iout6,25100) iscf,iorn(inde),bond(inde),  
     &                 gut(inde),eng(inde),denmax,1.0-area(inde)
                  write(iout6,25100) iscf,iorn(indn),bond(indn),  
     &                 gut(indn),eng(indn),demaxt(indn),1.0-area(indn)
               endif
            endif
         endif

         if (iscmc.eq.1) then
               call scmc (cw_orb,cw_coul,cw_exch,
     &           cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &           cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &           cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &           cw_sctch(i5b( 3)),cw_sctch(i5b( 4)),
     &           cw_sctch(i5b( 5)),cw_sctch(i5b( 6)),
     &           cw_sctch(i5b( 7)),cw_sctch(i5b( 8)),
     &           cw_sctch(i5b( 9)),cw_sctch(i5b(10)),
     &           cw_sctch(i5b(11)),cw_sctch(i5b(12)),
     &           cw_sctch(i5b(13)),cw_sctch(i5b(14)))
         endif

c     calculate and print various contributions to total energy
         if (iprint(50).ne.0) then
            call contrib (cw_orb,cw_coul,cw_exch,
     &           cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &           cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &           cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &           cw_sctch(i5b( 3)),cw_sctch(i5b( 4)))
         endif


c     Sometimes the convergence threshold (for energy and/or
c     normalization) is set too low and cannot be satisfied on a given
c     grid and as a result the SCF/SOR process continues in vain. To
c     avoid these excessive iterations the convergence process is
c     stopped if either denmax or dnomax has not decreased over a
c     given number (nlast) most recent iterations.

c     skip nscf2skip iterations before starting the search for
c     saturation; examine the last nenlast and nnolast iterations in case
c     of orbital energy and normalization, respectively

         if (iscf.gt.nscf2skip) then
            if (abs(denmax).ge.abs(ddmaxprev)) then
               noenergydec=noenergydec+1
            else
               ddmaxprev=denmax
               noenergydec=0
            endif

            if (abs(dnomax).ge.abs(dnmaxprev)) then
               nonormdec=nonormdec+1
            else
               dnmaxprev=dnomax
               nonormdec=0
            endif
c           print *,noenergydec,nonormdec,ddmaxprev,dnmaxprev
            if (noenergydec.gt.nenlast.or.nonormdec.gt.nnolast) then
               write(*,*) '          '
               write(*,*) '... solution cannot be further improved ...'
               goto 9999
            endif
         else
            ddmaxprev=demax(1)
            dnmaxprev=dnomax
         endif
         
c     check if the scf process is to be continued 

         if (abs(demax(1)).gt.diver) then
            write(*,*) '           '
            write(*,*) '... maximal orbital energy error is too ',
     &           'large ...'
            goto 9999
         endif

         if (iscf+1.gt.maxscf) then
            write(*,*) '          '
            write(*,*) '... scf iteration limit reached ... '
            goto 9999
         endif
         
         if (abs(denmax).lt.thren) then
            if (nthren.eq.0) then
               nthren=iscf
c              recalculate multipole moments
               call mpoleMom(cw_orb,cw_suppl,cw_sctch)
               imomen=1

               ddmax=denmax
               iasympt=norb
               
               do iorb=1,norb
                  itouch(iorb)=0
               enddo	
               goto 8888
            else
               if (nthren+1.ne.iscf) then
                  nthren=0
                  goto 8888
               else
                  write(*,*) '          '
                  write(*,*) '... orbital energy threshold reached ...'
                  goto 9999
               endif
            endif
         endif

         if ( thrno.ne.0.d0 ) then
            if (exlorb.eq.0.and.dnomax.lt.thrno) then
               write(*,*) '          '
               write(*,*) '... orbital normalization threshold',
     &              ' reached ...'
               goto 9999
            endif
         endif

 8888    continue

c        in case of OED calculations skip relaxing orbitals for which
c        either relaxation threshold has been achieved

         if (imethod.eq.2) then
            i=0
            do iorb=1,norb
               if (abs(demaxt(iorb)).lt.thren) ifix(iorb)=1
               if (ifix(iorb).eq.0) i=i+1
            enddo

            if (i.eq.0) then
               write(*,*) '          '
               write(*,*) '... orbital energy threshold ',
     &              'reached reached ...'
               goto 9999
            endif
         endif

       
         if (stop_x2dhf) then
            write(*,*) '          '
            write(*,*) '... stop_x2dhf detected ... '
            write(*,*) '... program is terminating ...'
            write(*,*)
            next=0
            goto 9999
         endif

c     recalculate multipole moments expansion coefficients 

         if (abs(ddmax/denmax).gt.facmul.or.
     &        abs(denmax/ddmax).gt.facmul) then
            call mpoleMom(cw_orb,cw_suppl,cw_sctch)
            imomen=1

            ddmax=denmax
            iasympt=norb
         
            do iorb=1,norb
               itouch(iorb)=0
            enddo	    
         endif
         
c        recalculate multipole coefficients and write functions to a disk
c        file if nobckup > 0
         
         if (nobckup.gt.0) then
            modv=mod(iscf,abs(nobckup))
            if (modv.eq.0) then
               if (imomen.eq.0.and.facmul.gt.0d0) then
                  call mpoleMom(cw_orb,cw_suppl,cw_sctch)
               endif
               imomen=0

               call writeDisk(cw_orb,cw_coul,cw_exch)
                  
c              calculate 'intermediate' total energy               
               if (iform.eq.1.or.iform.eq.3) then
                  if (islat.eq.0) then
                     call Etotal (cw_orb,cw_coul,cw_exch,
     &                 cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &                 cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &                 cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &                 cw_sctch(i5b( 3)),cw_sctch(i5b( 4)),
     &                 cw_sctch(i5b( 5)),cw_sctch(i5b( 6)),
     &                 cw_sctch(i5b( 7)),cw_sctch(i5b( 8)),
     &                 cw_sctch(i5b( 9)),cw_sctch(i5b(10)),
     &                 cw_sctch(i5b(11)),cw_sctch(i5b(12)),
     &                 cw_sctch(i5b(13)),cw_sctch(i5b(14)))
                  else
                     call EtotalDFT (cw_orb,cw_coul,cw_exch,
     &                 cw_suppl(i4b( 4)),cw_suppl(i4b( 5)),
     &                 cw_suppl(i4b(13)),cw_suppl(i4b(14)),
     &                 cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &                 cw_sctch(i5b( 3)),cw_sctch(i5b( 4)),
     &                 cw_sctch(i5b( 5)),cw_sctch(i5b( 6)),
     &                 cw_sctch(i5b( 7)),cw_sctch(i5b( 8)),
     &                 cw_sctch(i5b( 9)),cw_sctch(i5b(10)),
     &                 cw_sctch(i5b(11)),cw_sctch(i5b(12)),
     &                 cw_sctch(i5b(13)),cw_sctch(i5b(14)))
                  endif
                  call getCpuTime(time2)
                  ttoten=ttoten+(time2-time1)
               endif
               if (iprint16.eq.0) then
                  write(iout6,6100)  etot
               else
                  write(iout6,6101)  etot
               endif
            endif
         endif	  

c        to run a job in a bach mode (or in the background) with the
c        output redirected into a file and to be able to monitor every
c        single iteration one has to flush the output buffer to disk
      
         call flush(iout6)
      enddo
c     ----- end of scf loop	-----

09999 continue

c     write functions to disk file 

      if (nobckup.ge.0) then
         call writeDisk(cw_orb,cw_coul,cw_exch)
      endif
      return

 6100 format(1x,'total energy: ',e26.16)
 6101 format(1x,'total energy: ',e38.28)
15000 format(/'   scf  orbital',11x,'   energy  ',12x,'energy diff ',
     &         7x,'1-norm',9x,'overlap')
15100 format(i5,i4,1x,a8,a1,2x,e22.16,3e16.2)
15110 format(i5,i4,1x,a8,a1,3x,e22.16,2e16.2,i5)
25000 format(/'  iter.   orbital',24x,'   energy  ',19x,'energy diff.',
     &         7x,'1-norm')
25100 format(i5,i4,1x,a8,a1,2x,e44.32,3e16.2)

      end


      
