***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### expw86sup ###

c     Calculates exchange energy according to a formula of Parr and Wang
c     Yue (PRB 33 (1986) 8800)

      real*8 function expw86sup (wgt2,rho,grho,wk0,wk1)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const23=2.d0/3.d0,const43=4.d0/3.d0,
     &     const83=8.d0/3.d0,
     &     const115=1.d0/15.d0,a=1.296d0,b=14.d0,c=0.2d0)

      include 'commons8.inc'	
      dimension wgt2(*),rho(*),grho(*),wk0(*),wk1(*)

c      grho = nabla rho  nabla rho
c      |nabla rho| = sqrt(grho)

      const=(24.d0*pii*pii)**(-const23)
      do i=1,mxsize
         if (abs(rho(i)).lt.precis) then
            wk0(i)=0.d0
         else
            s2=const*grho(i)/rho(i)**const83
            s=sqrt(s2)
            fgga=(one+a*s2+b*s2*s2+c*s2*s2*s2)**const115

c Parr & Yue 1986  
            wk0(i)=rho(i)**const43*fgga

c Langreth-Mehl    
c           wk0(i)=rho(i)**const43*(one+1.521*0.0864*s2)

c GEA               
c           wk0(i)=rho(i)**const43*(one+0.0864*s2)

c LDA
c           wk0(i)=rho(i)**const43
         endif
      enddo	  

c     take care of F4 factor
      call multf4(wk0)

      expw86sup=dot(mxsize,wgt2,ione,wk0,ione)

      return
      end


