***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### expw91 ###

c     Calculates exchange energy according to a formula of Parr and Wang
c     Yue Phys. Rev. B 54, 16 533 (1996), Phys. Rev. B 45, 13 244
c     (1992).

      real*8 function expw91 (psi,wgt2,rhot,rhotup,rhotdown,
     &     grhot,grhotup,grhotdown,wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const43=4.d0/3.d0,
     &     const115=1.d0/15.d0)
      include 'commons8.inc'	

      dimension psi(*),wgt2(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),  
     &     wk5(*),wk6(*),wk7(*),  
     &     rhot(*),rhotup(*),rhotdown(*),
     &     grhot(*),grhotup(*),grhotdown(*)

      do i=1,mxsize
         rhotup(i)  =0.d0
         rhotdown(i)=0.d0
      enddo

c     calculate total densities due to up and down spins
      do iorb=1,norb
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         
         call exocc (iorb,ocup,ocdown)

         call prod2 (isiorb,psi(iborb),psi(iborb),wk1)
         call scal (isiorb,ocup,wk1,ione)
         
         call prod2 (isiorb,psi(iborb),psi(iborb),wk2)
         call scal (isiorb,ocdown,wk2,ione)
       
c        store total spin densities 
         call add(isiorb,wk1,rhotup)
         call add(isiorb,wk2,rhotdown)
      enddo

c     calculate (nabla rho nabla rho)
c FIXME
      call nfng (rhotup,rhotup,wk0,wk1,wk2,
     &     wk3,wk4,wk5,wk6,wk7)
      call copy(mxsize,wk7,ione,grhotup,ione)

      call nfng (rhotdown,rhotdown,wk0,wk1,wk2,
     &     wk3,wk4,wk5,wk6,wk7)
      call copy(mxsize,wk7,ione,grhotdown,ione)

      do i=1,mxsize
         rhotup(i)   =two*rhotup(i)
         rhotdown(i) =two*rhotdown(i)
         grhotup(i)  =four*grhotup(i)
         grhotdown(i)=four*grhotdown(i)
      enddo	  

c     total exchange energy is calculated as
c     Ex(rhoup,rhodown)=(1/2)Ex(2*rhoup)+(1/2)Ex(2*rhodown)

      expw91=fdften(alphaf)*
     &     (  half*expw91sup(wgt2,rhotup,grhotup,wk0,wk1) 
     &      + half*expw91sup(wgt2,rhotdown,grhotdown,wk0,wk1))

      return
      end
