***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### flypcs ###

c     Calculates (and returns in wk2) the correlation potential of Lee,
c     Yang and Parr (PRB 37 (1988) 785, eq.23, i.e. using the
c     closed-shell formula.

      subroutine flypcs (psi,f4,rhot,rhotup,rhotdown,
     &     grhot,grhotup,grhotdown,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'	

      dimension psi(*),f4(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),rhot(*),rhotup(*),rhotdown(*),
     &     grhot(*),grhotup(*),grhotdown(*)

      call zeroArray(mxsize,rhotup)
      call zeroArray(mxsize,rhotdown)

c     calculate total densities due to up and down spins
      do iorb=1,norb
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         
         call exocc (iorb,ocup,ocdown)

         if (ocup.ne.ocdown) then
            write(*,*) "Warning: This implementation of LYP potential",
     &           " is only valid for closed shell systems." 
            stop 'flypcs'
         endif

         call prod2 (isiorb,psi(iborb),psi(iborb),wk1)
         call scal (isiorb,ocup,wk1,ione)
         
         call prod2 (isiorb,psi(iborb),psi(iborb),wk2)
         call scal (isiorb,ocdown,wk2,ione)
       
c        store total densities 
         call add(isiorb,wk1,rhotup)
         call add(isiorb,wk2,rhotdown)
      enddo

      call copy(mxsize,rhotup,ione,rhot,ione)
      call add(mxsize,rhotdown,rhot)

      call flypsupcs (rhot,rhotup,rhotdown,grhot,grhotup,grhotdown,
     &     wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)

      call prod (mxsize,f4,wk7)
      call copy(mxsize,wk7,ione,wk2,ione)

      return
      end

