***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### flypsupcs ###

c     Calculates the correlation potential of Lee, Yang and Parr (PRB 37
c     (1988) 785, eq.23, i.e. using the closed-shell formula); it is
c     returned in wk10

      subroutine flypsupcs (rhot,rhotup,rhotdown,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7,wk8,wk9,wk10)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)

      parameter (const13=1.d0/3.d0,const23=2.d0/3.d0,const32=3.d0/2.d0,
     &     const34=3.d0/4.d0,
     &     const43=4.d0/3.d0,const53=5.d0/3.d0,
     &     const115=1.d0/15.d0,const83=8.d0/3.d0,
     &     const19=1.d0/9.d0,const118=1.d0/18.d0,
     &     const49=4.d0/9.d0)

c     coefficients of the Colle-Salveti formula 
      parameter(a=0.04918d0,b=0.132d0,c=0.2533d0,d=0.349d0)

      include 'commons8.inc'	

      dimension wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),wk8(*),wk(9),wk10(*),
     &     rhot(*),rhotup(*),rhotdown(*)

c     total density in rhot

c     nabla^2 rho (rhotup)
      call n2f (rhot,wk3,wk4,wk5,wk6)
      call copy (mxsize,wk6,ione,rhotup,ione)      

c     nabla rho nabla rho (rhotdown)
      call nfng (rhot,rhot,rhotdown,wk3,wk4,wk5,wk6,wk7,wk8,wk9)
      call copy (mxsize,wk9,ione,rhotdown,ione)      


      cf=0.3d0*(three*pii*pii)**const23

      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
            wk10(i)=0.d0
         else
            rh=rhot(i)**(-const13)
            f1=one/(one+d*rh)
            g0=rhot(i)**(-const53)*exp(-c*rh)
            g1=-const53+c/three*rhot(i)**(-const13)
            g2=g1+d/three*f1*rhot(i)**(-const13)
            g3=d*d*f1*f1*rhot(i)**(-const13) -d*f1-c

            wk10(i)=

c See documentation for the derivation of these terms  
c        term (1) 
     &           -a*d/three*rh*f1*f1-a*f1

c        term (2) 
     &           -a*b*cf*f1*g0*rhot(i)**(const53)*(g2+const83)

c        term (3) 
     &           -a*b*19.d0/18.d0*f1*g0*rhotup(i)

c        term (4) 
     &           -a*b*1.d0/72.d0*f1*g0*g2*
     &                    (42.d0*rhotup(i)+59.d0*rhotdown(i)/rhot(i))

c        term (5) 
     &           -a*b*7.d0/24.d0*f1*g0/rhot(i)*rhotdown(i)*
     &            ( g2*(d/three*f1*rh+g1-1.d0)
     &             +g3*rh/9.d0 )

         endif
      enddo	  

      return
      end

c ### flypsupcs1 ###
c
c     Calculates the correlation potential of Lee, Yang and Parr (PRB 37
c     (1988) 785, eq.23, i.e. using the closed-shell formula with the
c     gradient terms omitted). The potential is returned in wk10.
c

      subroutine flypsupcs1 (rhot,rhotup,rhotdown,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7,wk8,wk9,wk10)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const23=2.d0/3.d0)

c     coefficients of the Colle-Salveti formula 
      parameter(a=0.04918d0,b=0.132d0,c=0.2533d0,d=0.349d0)

      include 'commons8.inc'	

      dimension wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),wk8(*),wk(9),wk10(*),
     &     rhot(*),rhotup(*),rhotdown(*)


      cf=0.3d0*(three*pii*pii)**const23
c FIXME
      do i=1,mxsize
         wk10(i)=0.d0
      enddo

c     total density in rhot
c     F1 (wk0)

c     a simplified formula (no gradient terms)
      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
            wk10(i)=0.d0
         else
            rh=rhot(i)**(-const13)
            wk0(i)=one/(one+d*rh)
            wk10(i)=
c        term (1)
     &           -a*d/three*rhot(i)**(-const13)*wk0(i)*wk0(i)
     &           -a*wk0(i)
c        term (2)
     &           -a*b*cf*wk0(i)*exp(-c*rhot(i)**(-const13))*
     &           (one+(c+d*wk0(i))/three)
         endif
      enddo	  
      return
      end

