***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### fpw86sup ###

c     Calculates exchange potential according to a formula of Perdew and
c     Wang Yue (PRB 33(12) (1986) 8800) and returns it in grhot array

      subroutine fpw86sup (rhot,grhot,wk,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const43=4.d0/3.d0,
     &     const115=1.d0/15.d0,ifive=5)
      
      include 'commons8.inc'	

      dimension wk(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),rhot(*),grhot(*)

c     Calculate |(nabla rhot nabla rhot)| (wk)

      call nfng (rhot,rhot,wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)
      call copy(mxsize,wk7,ione,wk,ione)

      do i=1,mxsize
         wk(i)=sqrt(wk(i))
      enddo

c     calculate (nabla rho nabla |nabla rho|) (grhot)

      call nfng (rhot,wk,wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)
      call copy(mxsize,wk7,ione,grhot,ione)

c     calculate nabla^2 rho (wk7)

      call n2f (rhot,wk0,wk1,wk2,wk3)
      call copy(mxsize,wk3,ione,wk7,ione)
      call scal(mxsize,half,wk7,ione)

      fdftcoeff=fdftpot(alphaf)

      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
            s=0.d0
         else
c            akf=(three*pii*pii*rhot(i))**const13
            akf=(two*three*pii*pii*rhot(i))**const13
            s=wk(i)/(two*akf*rhot(i))
            t=wk7(i)*(two*akf)**(-2)/rhot(i)
            u=t/(two*akf)/rhot(i)*grhot(i)
c           write(*,'("s,t,u:",i4,3e14.4)') i,s,t,u
         endif

         w1=fdftcoeff*rhot(i)**const13

         if (abs(s).lt.precis) then
            wk0(i)=w1*const43*ff(s)
         else

C$$$         if (abs(u).gt.threshPW.or.abs(s).lt.precis) then
C$$$            wk0(i)=w1*const43*ff(s)
C$$$         else

c FIXME 
c
c           Check functional derivative (eq.24) since its second term causes
c           problems !!!  
c
c           12/2007 update 
c
c           The problems boils down to s^3 term which blows up
c           the potential

C$$$            term1=const43*ff(s)
C$$$            term2=-t/s*ffp(s)
C$$$            term3=-u*ffdp(s)
C$$$            term4=const43*s*s*s*ffdp(s)

c            w2=term1+term2+term3+term4
c            call i2numu(i,in,imu)
c            if (mod(i,ifive).eq.0) then
c               write(*,'("term1..term4 ",3i5,5e14.4)') 
c     &              i,in,imu,term1,term2,term3,term4,w2
c            endif

            w2=(const43*ff(s)-t/s*ffp(s)-(u-const43*s*s*s)*ffdp(s))
            wk0(i)=w1*w2
         endif
      enddo	  

      call copy(mxsize,wk0,ione,grhot,ione)

      return
      end

