***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### initSuppl ###

c     Initialises various supplementary arrays of case-dependent lengths
c     supported by cw_suppl (one-electron potentials, Jacobians,
c     integration and differentiation weights, etc)


      subroutine initSuppl (borb,bpot,d,e,f0,f1,f2,f3,f4,g,
     &		          wjac1,wjac2,wgt1,wgt2)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension borb(nni,mxnmu),bpot(nni,mxnmu),
     &          d(nni,mxnmu),e(nni,mxnmu),
     &		f0(nni,mxnmu),f1(nni,mxnmu),f2(nni,mxnmu),
     &          f3(nni,mxnmu),f4(nni,mxnmu),g(nni,mxnmu),
     &          wjac1(nni,mxnmu),wjac2(nni,mxnmu),
     &          wgt1(mxsize),wgt2(mxsize)

      dimension aa1(9),aa2(9),a1(9),a2(9)
      dimension dc1(9,7,9),dc2(9,7,9)

c     Coefficients of the first and second derivatives taken from from
c     the 8th-order Stirling interpolation formula

      data aa1/ 3.d0, -32.d0, 168.d0, -672.d0, 0.d0, 672.d0,
     &	     -168.d0,  32.d0,	-3.d0 /
      data aa2/  -9.d0,  128.d0, -1008.d0, 8064.d0, -14350.d0,
     &	       8064.d0,-1008.d0,   128.d0,   -9.d0 /

      precis1=precis

c     iaddr4(1)=borb
c     iaddr4(2)=bpot
c     iaddr4(3)=d
c     iaddr4(4)=e
c     iaddr4(5)=f0
c     iaddr4(6)=f1
c     iaddr4(7)=f2
c     iaddr4(8)=f3
c     iaddr4(9)=f4
c     iaddr4(10)=g
c     iaddr4(11)=wjac1
c     iaddr4(12)=wjac2
c     iaddr4(13)=wgt1
c     iaddr4(14)=wgt2

c     wjac1 - Jacobian for Rayleigh quotient
c     wjac2 - Jacobian for two-electron integrals

c     initialize arrays needed for the poisson equations

c FIXME
c     f4 - array of factors multiplying off-diagonal lagrange multipliers
c	   in the HF equation
c     off-diagonal l.m. must be multiplied by the same factor as
c     diagonal one i.e. f1 instead of f4

      if (ifefield.ne.0) then
         izz1=nint(z1)
         izz2=nint(z2)
         atw1=atweight(izz1)
         atw2=atweight(izz2)
         zcm=(-atw1+atw2)/(atw1+atw2)*r/2.d0
      endif
      
      wj1 =-r*pii/2.d0
      wj2 = r*r*pii/2.d0 
      xrr=half*r
      ib=ibmu(1)
      do ig=1,ngrids
         ie=iemu(ig)
         diag(ig)=-14350.d0/(5040.d0)*
     &        ( 1.d0/(hmu(ig)*hmu(ig))+1.d0/(hni*hni) )
         do i=ib,ie
            w1=vxi1(i)
            w2=vxi(i)*vxi(i)
            w9=xrr*vxi(i)

c           ssf is included in differentiation array dmu1
c           w4=vxi2(i)/ssf

            w4=vxi2(i)
            if(i.eq.1) then
               w5=0.d0
               w6=0.d0
            else
               w5= vxi(i)/vxi1(i)-2.0*vxi1(i)/vxi(i)
               w6= 1.d0/(vxi1(i)*vxi1(i))
            endif
            
            w8=-r*r*r*pii*vxi(i)/2.d0
            
            do j=1,nni
               w10=veta1(j)
               borb(j,i)=w4
               bpot(j,i)=w5
               d   (j,i)=veta2(j)
               if (w10.lt.precis) then
                  e(j,i)= 0.d0
               else
                  e(j,i)=-w6 - 1.d0/(w10*w10)
               endif

               w7=w2-veta(j)*veta(j)
               f1(j,i)= r*r*w7/2.d0
               if (ipot.eq.0) then
c                 point nuclei - Coulomb potential
                  f0(j,i)=r*(vxi(i)*(z1+z2)+veta(j)*(z2-z1))
               elseif (ipot.eq.1) then
c                 finite nuclei - Fermi model
                  z1t=zz1(i,j)
                  z2t=zz2(i,j)
                  f0(j,i)=r*(vxi(i)*(z1t+z2t)+veta(j)*(z2t-z1t))
               elseif (ipot.eq.2) then
c                 finite nuclei - Gauss model
                  z1t=zz1g(i,j)
                  z2t=zz2g(i,j)
                  f0(j,i)=r*(vxi(i)*(z1t+z2t)+veta(j)*(z2t-z1t))
               elseif (ipot.eq.3) then
c                 OED - 3D Coulomb potential reduced to 2D external
c                 potential must be multiplied by -f1 and added to f0
                  z=(r/2.d0)*vxi(i)*veta(j)
                  xxplusyy2=(r/2.d0)*vxi1(i)*veta1(j)
                  f0(j,i)=-f1(j,i)*poth3(z,xxplusyy2,mpot,v0pot,apot,
     &                 precis1)
c                  print *,i,j,z,xxplusyy2,apot,mpot,f0(j,i)
               elseif (ipot.eq.4) then
c                 OED - Kramers-Henneberger potential
                  z=(r/2.d0)*vxi(i)*veta(j)
                  xxplusyy2=(r/2.d0)*vxi1(i)*veta1(j)
                  f0(j,i)=-f1(j,i)*potkh(z,xxplusyy2,mpot,epspot,ompot,
     &                 v0pot,apot,dble(nsimp),precis1) 
               elseif (ipot.eq.5) then
c                 OED - Green, Sellin, Zachor model potential 
                  z1t=zgsz1(i,j)
                  z2t=zgsz2(i,j)
                  f0(j,i)=r*(vxi(i)*(z1t+z2t)+veta(j)*(z2t-z1t))
               elseif (ipot.eq.55) then
c                 OED - Green, Sellin, Zachor model potential + finite Gauss nucleus model 
                  z1t=zgsz1g(i,j)
                  z2t=zgsz2g(i,j)
                  f0(j,i)=r*(vxi(i)*(z1t+z2t)+veta(j)*(z2t-z1t))

               elseif (ipot.eq.9) then
c                 harmonic potential (Hook's atom, ahmonium) located at A centre
                  r1=(r/2.d0)*(vxi(i)+veta(j))
                  f0(j,i) =-hook/two*(r1*r1)*f1(j,i)

               elseif (ipot.eq.10) then
c                 harmonic potential (x^2+y^2+z^2)
                  z=(r/2.d0)*vxi(i)*veta(j)
                  xy2=(r*r/4.d0)*(vxisq(i)-one)*(one-vetasq(j))
                  f0(j,i) =  -(xy2+z*z)*r*r*half*(vxisq(i)-vetasq(j))

               endif

               if (ifefield.eq.1) then

c                 when external electric field is present (z-zcm)*E 
c                 has to be added (times -f1)

                  z=(r/2.d0)*vxi(i)*veta(j)
                  if (abs(z).lt.zcutoff) then
                     f0(j,i)=f0(j,i)-ffield*(z-zcm)*f1(j,i)
                  else
                     f0(j,i)=f0(j,i)-ffield*(z-zcm)*
     &                    exp(zcutoff-abs(z))*f1(j,i)
                  endif 

               elseif (ifefield.eq.2) then
c FIXME
c                 if electric field gradient is nonzero the interaction 
c                 energy is -(Q_z/4)*(dE/dz) 

c                 if electric field gradient is nonzero the interaction 
c                 energy is  -R^2/3 P(2,0)*(dE/dz) (Greiner, Electrodynamics, p.91)

                  imu=i
                  in=j
                  z=(r/2.d0)*vxi(imu)*veta(in)-zcm
                  xxplusyy=((r/2.d0)*vxi1(imu)*veta1(in))**2
                  rr=sqrt(xxplusyy+z*z)
                  if (abs(rr).lt.precis) then
                     costh=0.d0
                  else
                     costh=z/rr
                  endif
                  
                  wktmp=rr**2*plegendg(itwo,izero,costh)
                  z=(r/2.d0)*vxi(i)*veta(j)
                  if (abs(z).lt.zcutoff) then
                     f0(j,i)=f0(j,i)-fgrad*wktmp*f1(j,i)/3.d0
                  else
                     f0(j,i)=f0(j,i)-fgrad*wktmp*f1(j,i)/3.d0
     &                    *exp(zcutoff-abs(z))
                  endif
               endif

               if (iharm2xy.eq.1) then
c                 external three-dimensional harmonic oscilator in x-y-z coordinates 
c                 potential must be multiplied by -f1 and added to f0
                  imu=i
                  in=j
                  f0(j,i)=f0(j,i)-f1(in,imu)*
     &                 harm2xy*harm2xy/2.d0*xrr*xrr*
     &                 (vxi(i)*vxi(i)+veta(j)*veta(j)-1.d0)

               elseif (iharm2xy.eq.2) then
c                 external two-dimensional harmonic oscilator in x-y coordinates 
c                 potential must be multiplied by -f1 and added to f0
                  imu=i
                  in=j
                  f0(j,i)=f0(j,i)-
     &                 harm2xy*harm2xy/2.d0*xrr*xrr*f1(in,imu)*
     &                 vxi1(imu)*vxi1(imu)*veta1(in)*veta1(in)

               endif

               if(i.eq.1) then
                  f2(j,i)=0.d0
               else
                  f2(j,i)=-r*w7/vxi(i)
               endif
               f3(j,i)=-2.d0/w2
               g (j,i)= w8*w7
               f4(j,i)= w9
               
c              Integration needs hmu*hni but the integration routine works
c              for hni*hni. That is why ssf must be inserted here.

c              wjac1(j,i)=wj1*w1*w5/ssf(ig)
c              wjac2(j,i)=wj2*w1*w5*(w3+w6)/(ssf(ig)*w0)

               wjac1(j,i)=wj1*w1*w10
               wjac2(j,i)=wj2*w1*w10*(w1*w1+w10*w10)/vxi(i)
            enddo
         enddo
         ib=ie+1
c        enddo over ig
      enddo

c     initialize vector of integration weights
c     (7-point formula	Abramovic 25.4.16.)

      wmup=0.d0
      do ig=1,ngrids
         ib=ibmu(ig)
         ie=iemu(ig)
         w1 =hmu(ig)/140.d0
         wmu(ib)=wmup+w1*41.d0
         do i=ib+1,ie,6
            wmu(i  )=w1*216.d0
            wmu(i+1)=w1* 27.d0
            wmu(i+2)=w1*272.d0
            wmu(i+3)=w1* 27.d0
            wmu(i+4)=w1*216.d0
            if (i+5.eq.ie) then
               wmu(i+5)=w1*41.d0
            else
               wmu(i+5)=2.d0*w1*41.d0
            endif
         enddo
         wmup=wmu(ie)
      enddo

      w1    =hni/140.d0
      wni(1)=w1*41.d0
      do i=2,nni,6
         wni(i  )=w1*216.d0
         wni(i+1)=w1* 27.d0
         wni(i+2)=w1*272.d0
         wni(i+3)=w1* 27.d0
         wni(i+4)=w1*216.d0
         if (i+5.eq.nni) then
            wni(i+5)=w1*41.d0
         else
            wni(i+5)=2.d0*w1*41.d0
         endif
c        enddo over i
      enddo

c     multiply Jacobians for one and two-electron integrals by elements
c     of wmu array

      do i=1,mxnmu
         w1=wmu(i)
         do j=1,nni
            wjac1(j,i)=w1*wjac1(j,i)
            wjac2(j,i)=w1*wjac2(j,i)
         enddo
      enddo

c     prepare superarrays with the integration weights over ni variable
c     and with jacobians included

      do i=1,mxnmu
         ii=(i-1)*nni
         do j=1,nni
            wgt1(ii+j)=wni(j)*wjac1(j,i)
            wgt2(ii+j)=wni(j)*wjac2(j,i)
         enddo
      enddo

c     Initialize arrays needed for differentiation based on the 9-point
c     interpolation formula

c     Initialize dni1, dni2, dmu1 and dmu2

      w1=1.d0/(840.d0)
      w2=1.d0/(5040.d0)

      do ig=1,ngrids
         do  i=1,4
            dmu2(i,ig)=w2*aa2(i)/(hmu(ig)*hmu(ig))
            dmu1(i,ig)=w1*aa1(i)/hmu(ig)
         enddo
      enddo

      do i=1,4
         dni2(i)=w2*aa2(i)/(hni*hni)
         dni1(i)=w1*aa1(i)/hni
      enddo

c     Initialize arrays needed for fast (matrix times matrix) evaluation
c     of the first and second dervivative terms in the Laplasian (dmu, dnu) and
c     the first derivatives for gradients (d1mu, d1nu)

c     derivatives over mu variable
      ib=1
      do ig=1,ngrids
         do k=1,9
            a2(k)=w2*aa2(k)/(hmu(ig)*hmu(ig))
            a1(k)=w1*aa1(k)/hmu(ig)
         enddo 

         ie=iemu(ig)
         do i=ib,ie
            do k=1,9
               dmu(k,i)=a2(k)+borb(1,i)*a1(k)
               d1mu(k,i)=a1(k)
            enddo
         enddo
         ib=ie+1
      enddo

c     derivatives over nu variable
      do k=1,9
         a2(k)=w2*aa2(k)/(hni*hni)
         a1(k)=w1*aa1(k)/hni
      enddo

      do i=1,nni
         do k=1,9
            dni(k,i)=a2(k)+d(i,1)*a1(k)
            d1ni(k,i)=a1(k)
         enddo
      enddo

c FIXME obsolete in version 2.0
      if (ngrids.ne.1) then
         
c        initialize arrays dc1 and dc2

c        in order to check lagrpol use two ajasent grids with equal step size 
c        and compare coefficients with those of the Stirling formulae

         call lagrpolq (dc1,dc2)

c        modify dmu array at the grid boundary region
c        derivative coeff. from 8th-order Lagrange interpolation formula
c        are stored in dc2(ngbound,imu,k) and dc1(ngbound,imu,k),
c        where ngbound is the number of 
c        the grid boundaries (1 for 1-2, 2 for 2-3 etc), imu is one of the 7 
c        intergrid mu values and k points to a derivative coefficient.

         do ig=1,ngrids-1
            ib=iemu(ig)-3
            ie=iemu(ig)+3
            imu=0
            do i=ib,ie
               imu=imu+1
               do k=1,9
c                 a2(k)=dc2(ig,imu,k)/(hmu(ig)*hmu(ig))
c                 a1(k)=dc1(ig,imu,k)/hmu(ig)
                  a2(k)=dc2(ig,imu,k)
                  a1(k)=dc1(ig,imu,k)
               enddo 
               
               do k=1,9
                  dmu(k,i)=a2(k)+borb(1,i)*a1(k)
               enddo
            enddo
         enddo
      endif

      return
      end
