***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### nmucalc ###

c     Calculates nmu for given nni and practical infinity values (a
c     single grid case is assumed) and adjusts nmu to be of the form
c     6k+1 and 5k+5+1 (if mcsor is chosen)

      function nmucalc(n)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      parameter (k5=5, k6=6)

c     Since in this version of the program the 7 point integration
c     formula is used the number of mesh points in the nu and mu
c     variables must be of the form 6*n+1

c     Another restriction has to be imposed on the number of points in the
c     mu variable if the mcsor scheme is to be used: nmu must be
c     of the form kN+k+1, where k=iorder/2+1 for each grid (in this program
c     iorder, i.e. the order of descritization is 8) 

c     mxnmu --  maximum no. of grid points in mu variable
c     hni - step in ni variable
c     hmu - step in mu variable
      
      ig=1

      if (n.eq.0) then

c        determine step size in nmu variable so as hmu is approximately
c        equal to hni

         hnu=pii/dble(nni-1)

         xmi0=2.d0*rgrid(ig)/r
         xmi0=log(xmi0+sqrt(xmi0*xmi0-1.d0))
c        hmu(1)=xmi0/dble(mxnmu-1)
         mxnmu=xmi0/hnu+1
         nmu(ig)=mxnmu
         n=nmu(1)
      endif

c     adjust, if necessary, the number of grid points in mu variable

 12   nk=(n-1)/k6
      if (k6*nk.ne.n-1) then
         n=n-1
         goto 12
      endif

      ntemp=n
      if (ipoiss.eq.2.and.iorder(ngrids).eq.3) then
 22      nk=(n-6)/k5
         if (k5*nk.ne.n-k5-1) then
            n=n-1
            goto 22
         endif
      endif

      if (n.gt.1.and.n.ne.ntemp) goto 12

      nmu(ig)=n
      mxnmu=nmu(ig)
      
c     calculate hmu(1) for the adjusted value of mxnmu
      xmi0=2.d0*rgrid(ig)/r
      xmi0=log(xmi0+sqrt(xmi0*xmi0-1.d0))
      hmu(1)=xmi0/dble(mxnmu-1)

      nmucalc=mxnmu
      
      return
      end
