***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### orbMCSOR ###  

c     Evaluates the Fock potential for a given orbital, sets up the
c     right-hand side of the Poisson equation for that orbital and
c     performs a few MCSOR iterations.

      subroutine orbMCSOR (iorb,cw_sor,psi,pot,excp,b,d,e,
     &     f0,f1,f2,f4,wgt2,
     &     lhs,rhs,wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9,fock1,fock2)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      integer*4 cw_sor
      real*8 lhs
      include 'commons8.inc'

      dimension psi(*),pot(*),excp(*),cw_sor(*),
     &     b(*),d(*),e(*),f0(*),f1(*),f2(*),f4(*),wgt2(*),
     &     lhs(*),rhs(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),wk8(*),wk9(*),fock1(*),fock2(*)

c     prepare right and left-hand side of the Poisson equation
c     include the diagonal part of the differentiation operator in lhs

      if (islat.eq.0) then
c         call fock (iorb,psi,pot,excp,e,f0,f1,f2,f4,wk0,wk1,wk2,wk3)
         call fock (iorb,psi,pot,excp,e,f0,f1,f2,f4,fock1,rhs,wk1,wk2)
      else
c        DFT exchange approximation is used
         call fockDFT(iorb,psi,pot,excp,e,f0,f1,f2,f4,lhs,rhs,
     &        wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7,wk8,wk9,wk10,wk11)
      endif

c     start the sor iterations
c     fill wk2 array

      isym=isymOrb(iorb)
      iborb=i1b(iorb)
      
      do itr1=1,maxsor1	  
         do ig=1,i1ng(iorb)
            muoffs=0
            omega=ovforb(ig)
            omega1=1.d0-omega
            ioffst=ioffs(ig)	  	  
            ioffs1=ioffst+1	  
            iborb1=iborb+ioffst

            do i=1,4
               dmu2t(i)=dmu2(i,ig)
               dmu1t(i)=dmu1(i,ig)
            enddo
            
            do i=1,ngsize(ig)
               lhs(ioffst+i)=fock1(ioffst+i)+diag(ig)
            enddo	    

            if (i1ng(iorb).eq.1) then 
c              icase=1
               ifill=1
               ngrd1 =ingr1(1,ig)
               ngrd6a=ingr1(2,ig)
               ngrd6b=ingr1(3,ig)
               ngrd7 =ingr1(4,ig)	  	  

c              initialize aorb array needed to evaluate orbital values in
c              the asymptotic region

               nmut=i1mu(iorb)
               do i=nmut-4,nmut
                  ii=(i-1)*nni
                  do j=1,nni
                     ij=ii+j
                     if (f1(ij).eq.0.d0) then
                        wk1(ij)=0.d0
                     else
                        wk1(ij)=lhs(ij)/f1(ij)
                     endif
                  enddo
               enddo

               call orbAsymptDet (nmut,iorb,wk3,wk1)
               call putin (nni,nmu(ig),isym,psi(iborb),wk2)

c               maxsor=maxsororb(iorb)
               do itr2=1,maxsororb(iorb)	  		
                  call mcsor(
     &                 wk2,lhs(ioffs1),rhs(ioffs1),b(ioffs1),d(ioffs1),
     &                 cw_sor(iadext(ig)),cw_sor(iadnor(ig)),
     &                 cw_sor(iadex1(ig)),cw_sor(iadex2(ig)),
     &                 cw_sor(iadex3(ig)))
               enddo
               call putout (nni,nmu(ig),psi(iborb),wk2)
               call orbAsymptSet (nmut,psi(iborb),wk3)				
            else
               if (ig.eq.1) then 
c                 icase=2
                  ifill=2
                  ngrd1 =ingr2(1,ig)
                  ngrd6a=ingr2(2,ig)
                  ngrd6b=ingr2(3,ig)
                  ngrd7 =ingr2(4,ig)	  	  	    
                  
                  call putin2 (nni,nmu(ig),psi(iborb),wk2)
                  do itr2=1,maxsororb(iorb)	  		
                     call mcsor 
     &                    (wk2,lhs(ioffs1),rhs(ioffs1),b(ioffs1),
     &                    d(ioffs1),
     &                    cw_sor(iadext(ig+ngrids)),
     &                    cw_sor(iadnor(ig+ngrids)),
     &                    cw_sor(iadex1(ig+ngrids)),
     &                    cw_sor(iadex2(ig+ngrids)),
     &                    cw_sor(iadex3(ig+ngrids)))
                  enddo     
                  call putout (nni,nmu(ig),psi(iborb),wk2)     
               elseif (ig.ne.1.and.ig.ne.i1ng(iorb)) then 
c                 icase=2
                  ifill=3
                  ngrd1 =ingr2(1,ig)
                  ngrd6a=ingr2(2,ig)
                  ngrd6b=ingr2(3,ig)
                  ngrd7 =ingr2(4,ig)	  	  	    
                  
                  muoffs=iemu(ig-1)-1
                  call putin3 (nni,nmu(ig),psi(iborb),wk2)
                  do itr2=1,maxsororb(iorb)	  		
                     call mcsor 
     &                    (wk2,lhs(ioffs1),rhs(ioffs1),b(ioffs1),
     &                    d(ioffs1),
     &                    cw_sor(iadext(ig+ngrids)),
     &                    cw_sor(iadnor(ig+ngrids)),
     &                    cw_sor(iadex1(ig+ngrids)),
     &                    cw_sor(iadex2(ig+ngrids)),
     &                    cw_sor(iadex3(ig+ngrids)))
                  enddo     
                  call putout34 (nni,nmu(ig),psi(iborb),wk2)     
                  muoffs=0
               elseif (ig.eq.i1ng(iorb)) then 
                  
c                 initialize aorb array needed to evaluate orbital values in
c                 the asymptotic region
                  
                  nmut=i1mu(iorb)
                  do i=nmut-4,nmut
                     ii=(i-1)*nni
                     do j=1,nni
                        ij=ii+j
                        if (f1(ij).eq.0.d0) then
                           wk1(ij)=0.d0
                        else
                           wk1(ij)=lhs(ij)/f1(ij)
                        endif
                     enddo
                  enddo
c                  call iniaorb (nmut,wk0,wk1)
                  
                  ifill=4
                  ngrd1 =ingr1(1,ig)
                  ngrd6a=ingr1(2,ig)
                  ngrd6b=ingr1(3,ig)
                  ngrd7 =ingr1(4,ig)	  	  	    
                  
                  muoffs=iemu(ig-1)-1
                  
                  call putin4 (nni,nmu(ig),psi(iborb),wk2)
                  do itr2=1,maxsororb(iorb)	  		
                     call mcsor 
     &                    (wk2,lhs(ioffs1),rhs(ioffs1),b(ioffs1),
     &                    d(ioffs1),
     &                    cw_sor(iadext(ig)),cw_sor(iadnor(ig)),
     &                    cw_sor(iadex1(ig)),cw_sor(iadex2(ig)),
     &                    cw_sor(iadex3(ig)))
                  enddo
                  call putout34 (nni,nmu(ig),psi(iborb),wk2)     
c                  call asymorb (nmut,psi(iborb),wk0)	
                  muoffs=0	    
               endif
            endif
         enddo
      enddo
      
      return
      end

***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### mcsor1 ###
c
c     Performs one iteration of the sor process applied to
c     the equation (diagonal part of the differentiation operator, i.e.
c     the coefficient multiplying fun(ni,mi) is included in lhs array)
c
c     { d2(mi) + b(mi)*d1(mi) + d2(ni) + d(ni)*d1(ni)
c
c                     + lhs(ni,mi) } fun(ni,mi) = rhs(ni,mi)

      subroutine mcsor1 (fun,lhs,rhs,b,d,
     &     indx1,indx2,indx6a,indx6b,indx7,maxsor)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      real*8	lhs
      include 'commons8.inc'
      dimension fun(*),lhs(*),rhs(*),b(*),d(*),
     &     indx1(*),indx2(*),indx6a(*),indx6b(*),indx7(*)	

c     indx1 (indx2) array elements point to the particular (consequtive)
c     element of fun (lhs, rhs, b and d) to be relaxed
c
c     first relax the inner points -- region i (formula a1)
c
c     ipoiss=2 -- inner variables are relaxed in 5 sweeps each relaxing
c     every 5th variable.
c

c     directives cdir$ ivdep and cdir$ parallel cannot be used to force
c     automatic parallelization of the loops. Open MP directives have to
c     be used instead
      
!$OMP  PARALLEL  
!$OMP&    SHARED(indx1,indx2,maxsor,nni1,nni2,nni3,nni4,b,rhs,lhs,
!$OMP&             dmu2t,dmu1t,dni2,dni1,fun,omega,omega1)
      
!$OMP&    PRIVATE(i,ij,ik,ddmi2,ddmi1,ddni2,ddni1,cc)
!$OMP&    NUM_THREADS (5)

!$OMP SECTIONS   

!$OMP SECTION            

      do k=1,maxsor
      do i=1,ngrd1,5
         ij=indx1(i)
         ik=indx2(i)

         ddmi2=dmu2t(1) * ( fun(ij-nni4) + fun(ij+nni4) )+
     &         dmu2t(2) * ( fun(ij-nni3) + fun(ij+nni3) )+
     &         dmu2t(3) * ( fun(ij-nni2) + fun(ij+nni2) )+
     &         dmu2t(4) * ( fun(ij-nni1) + fun(ij+nni1) )
         
         ddmi1=dmu1t(1) * ( fun(ij-nni4) - fun(ij+nni4) )+
     &         dmu1t(2) * ( fun(ij-nni3) - fun(ij+nni3) )+
     &         dmu1t(3) * ( fun(ij-nni2) - fun(ij+nni2) )+
     &         dmu1t(4) * ( fun(ij-nni1) - fun(ij+nni1) )
         
         ddni2=dni2(1) * ( fun(ij-   4) + fun(ij+   4) )+
     &         dni2(2) * ( fun(ij-   3) + fun(ij+   3) )+
     &         dni2(3) * ( fun(ij-   2) + fun(ij+   2) )+
     &         dni2(4) * ( fun(ij-   1) + fun(ij+   1) )
         
         ddni1=dni1(1) * ( fun(ij-   4) - fun(ij+   4) )+
     &         dni1(2) * ( fun(ij-   3) - fun(ij+   3) )+
     &         dni1(3) * ( fun(ij-   2) - fun(ij+   2) )+
     &         dni1(4) * ( fun(ij-   1) - fun(ij+   1) )
         
         cc=ddmi2 + b(ik)*ddmi1 + ddni2 + d(ik)*ddni1
         fun(ij) = omega * (rhs(ik)-cc)/lhs(ik)+ omega1 * fun(ij)
      enddo
      enddo
      
!$OMP SECTION            

      do k=1,maxsor
      do i=2,ngrd1,5
         ij=indx1(i)
         ik=indx2(i)

         ddmi2=dmu2t(1) * ( fun(ij-nni4) + fun(ij+nni4) )+
     &         dmu2t(2) * ( fun(ij-nni3) + fun(ij+nni3) )+
     &         dmu2t(3) * ( fun(ij-nni2) + fun(ij+nni2) )+
     &         dmu2t(4) * ( fun(ij-nni1) + fun(ij+nni1) )
         
         ddmi1=dmu1t(1) * ( fun(ij-nni4) - fun(ij+nni4) )+
     &         dmu1t(2) * ( fun(ij-nni3) - fun(ij+nni3) )+
     &         dmu1t(3) * ( fun(ij-nni2) - fun(ij+nni2) )+
     &         dmu1t(4) * ( fun(ij-nni1) - fun(ij+nni1) )

         ddni2=dni2(1) * ( fun(ij-   4) + fun(ij+   4) )+
     &         dni2(2) * ( fun(ij-   3) + fun(ij+   3) )+
     &         dni2(3) * ( fun(ij-   2) + fun(ij+   2) )+
     &         dni2(4) * ( fun(ij-   1) + fun(ij+   1) )
         
         ddni1=dni1(1) * ( fun(ij-   4) - fun(ij+   4) )+
     &         dni1(2) * ( fun(ij-   3) - fun(ij+   3) )+
     &         dni1(3) * ( fun(ij-   2) - fun(ij+   2) )+
     &         dni1(4) * ( fun(ij-   1) - fun(ij+   1) )

         cc=ddmi2 + b(ik)*ddmi1 + ddni2 + d(ik)*ddni1
         fun(ij) = omega * (rhs(ik)-cc)/lhs(ik)+ omega1 * fun(ij)
      enddo
      enddo

!$OMP SECTION            

      do k=1,maxsor
      do i=3,ngrd1,5
         ij=indx1(i)
         ik=indx2(i)

         ddmi2=dmu2t(1) * ( fun(ij-nni4) + fun(ij+nni4) )+
     &         dmu2t(2) * ( fun(ij-nni3) + fun(ij+nni3) )+
     &         dmu2t(3) * ( fun(ij-nni2) + fun(ij+nni2) )+
     &         dmu2t(4) * ( fun(ij-nni1) + fun(ij+nni1) )

         ddmi1=dmu1t(1) * ( fun(ij-nni4) - fun(ij+nni4) )+
     &         dmu1t(2) * ( fun(ij-nni3) - fun(ij+nni3) )+
     &         dmu1t(3) * ( fun(ij-nni2) - fun(ij+nni2) )+
     &         dmu1t(4) * ( fun(ij-nni1) - fun(ij+nni1) )

         ddni2=dni2(1) * ( fun(ij-   4) + fun(ij+   4) )+
     &         dni2(2) * ( fun(ij-   3) + fun(ij+   3) )+
     &         dni2(3) * ( fun(ij-   2) + fun(ij+   2) )+
     &         dni2(4) * ( fun(ij-   1) + fun(ij+   1) )

         ddni1=dni1(1) * ( fun(ij-   4) - fun(ij+   4) )+
     &         dni1(2) * ( fun(ij-   3) - fun(ij+   3) )+
     &         dni1(3) * ( fun(ij-   2) - fun(ij+   2) )+
     &         dni1(4) * ( fun(ij-   1) - fun(ij+   1) )

         cc=ddmi2 + b(ik)*ddmi1 + ddni2 + d(ik)*ddni1
         fun(ij) = omega * (rhs(ik)-cc)/lhs(ik)+ omega1 * fun(ij)
      enddo
      enddo

!$OMP SECTION            

      do k=1,maxsor
      do i=4,ngrd1,5
         ij=indx1(i)
         ik=indx2(i)

         ddmi2=dmu2t(1) * ( fun(ij-nni4) + fun(ij+nni4) )+
     &         dmu2t(2) * ( fun(ij-nni3) + fun(ij+nni3) )+
     &         dmu2t(3) * ( fun(ij-nni2) + fun(ij+nni2) )+
     &         dmu2t(4) * ( fun(ij-nni1) + fun(ij+nni1) )

         ddmi1=dmu1t(1) * ( fun(ij-nni4) - fun(ij+nni4) )+
     &         dmu1t(2) * ( fun(ij-nni3) - fun(ij+nni3) )+
     &         dmu1t(3) * ( fun(ij-nni2) - fun(ij+nni2) )+
     &         dmu1t(4) * ( fun(ij-nni1) - fun(ij+nni1) )
        
         ddni2=dni2(1) * ( fun(ij-   4) + fun(ij+   4) )+
     &         dni2(2) * ( fun(ij-   3) + fun(ij+   3) )+
     &         dni2(3) * ( fun(ij-   2) + fun(ij+   2) )+
     &         dni2(4) * ( fun(ij-   1) + fun(ij+   1) )

         ddni1=dni1(1) * ( fun(ij-   4) - fun(ij+   4) )+
     &         dni1(2) * ( fun(ij-   3) - fun(ij+   3) )+
     &         dni1(3) * ( fun(ij-   2) - fun(ij+   2) )+
     &         dni1(4) * ( fun(ij-   1) - fun(ij+   1) )

         cc=ddmi2 + b(ik)*ddmi1 + ddni2 + d(ik)*ddni1
         fun(ij) = omega * (rhs(ik)-cc)/lhs(ik)+ omega1 * fun(ij)
      enddo
      enddo

!$OMP SECTION            

      do k=1,maxsor
      do i=5,ngrd1,5
         ij=indx1(i)
         ik=indx2(i)

         ddmi2=dmu2t(1) * ( fun(ij-nni4) + fun(ij+nni4) )+
     &        dmu2t(2) * ( fun(ij-nni3) + fun(ij+nni3) )+
     &        dmu2t(3) * ( fun(ij-nni2) + fun(ij+nni2) )+
     &        dmu2t(4) * ( fun(ij-nni1) + fun(ij+nni1) )

         ddmi1=dmu1t(1) * ( fun(ij-nni4) - fun(ij+nni4) )+
     &        dmu1t(2) * ( fun(ij-nni3) - fun(ij+nni3) )+
     &        dmu1t(3) * ( fun(ij-nni2) - fun(ij+nni2) )+
     &        dmu1t(4) * ( fun(ij-nni1) - fun(ij+nni1) )

         ddni2=dni2(1) * ( fun(ij-   4) + fun(ij+   4) )+
     &        dni2(2) * ( fun(ij-   3) + fun(ij+   3) )+
     &        dni2(3) * ( fun(ij-   2) + fun(ij+   2) )+
     &        dni2(4) * ( fun(ij-   1) + fun(ij+   1) )

         ddni1=dni1(1) * ( fun(ij-   4) - fun(ij+   4) )+
     &        dni1(2) * ( fun(ij-   3) - fun(ij+   3) )+
     &        dni1(3) * ( fun(ij-   2) - fun(ij+   2) )+
     &        dni1(4) * ( fun(ij-   1) - fun(ij+   1) )

         cc=ddmi2 + b(ik)*ddmi1 + ddni2 + d(ik)*ddni1
         fun(ij) = omega * (rhs(ik)-cc)/lhs(ik)+ omega1 * fun(ij)
      enddo
      enddo

!$OMP END SECTIONS

!$OMP END PARALLEL

c   determine values at the bondary points from extrapolation
      
      if (isym.eq.1) then
         if (ifill.eq.1.or.ifill.eq.2) then
            do i=1,ngrd7
               ij=indx7(i)
               fun(ij)=exeven(1)*fun(ij+nni1)+exeven(2)*fun(ij+nni2)+
     &              exeven(3)*fun(ij+nni3)+exeven(4)*fun(ij+nni4)+
     &              exeven(5)*fun(ij+nni5)
            enddo
         endif     

         do i=1,ngrd6a
            ij=indx6a(i)
            fun(ij)=exeven(1)*fun(ij+1)+exeven(2)*fun(ij+2)+
     &           exeven(3)*fun(ij+3)+exeven(4)*fun(ij+4)+
     &           exeven(5)*fun(ij+5)
         enddo

         do i=1,ngrd6b
            ij=indx6b(i)
            fun(ij)=exeven(1)*fun(ij-1)+exeven(2)*fun(ij-2)+
     &           exeven(3)*fun(ij-3)+exeven(4)*fun(ij-4)+
     &           exeven(5)*fun(ij-5)
         enddo
      else
         if (ifill.eq.1.or.ifill.eq.2) then
            do i=1,ngrd7
               ij=indx7(i)
               fun(ij)=0.d0
            enddo
         endif

         do i=1,ngrd6a
            ij=indx6a(i)
            fun(ij)=0.d0
        enddo

        do i=1,ngrd6b
           ij=indx6b(i)
           fun(ij)=0.d0
        enddo
      endif

      return
      end
