***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### x2dhf ###
c
c     This is the main routine of the 2DHF program.
c     Dimensions of working arrays have to be defined either statically 
c     or dynamically, i.e. based on input data (see prep_src).
                                                                      
      PROGRAM  x2dhf
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)

c     FIXME the following variables (used as pointers) must be 4-byte long
c     when 32-bit machine and 64-bit integers are used

      integer*4 cw_orb_add(1),cw_coul_add(1),cw_exch_add(1),
     &		cw_suppl_add(1),cw_sctch_add(1),cw_sor_add(1), 
     &		i4,i8,offset1,offset2,offset3,offset4,
     &		offset5,offset6,size
      include 'commons8.inc'

      character*1 str(80)
      equivalence(datetime,str(1))

      data i4,i8/4,8/

 100  continue
      next=0
c     set number of additional arrays
      nsuppl=14
      nsctch=14

      call separator
      call printBanner
      call separator

      call  getDateTime(datetime)
      dtstart=datetime

c     determine and set precision constants
      call setPrecision

c     set default values of various parameters
      call setDefaults

c     no and nons denote the number of all the orbitals and number 
c     of non sigma type orbitals, respectively. ni and mu correspond to
c     the number of grid points in ni and mu variables.

c     read input data and echo input parameters
c     open input/output files

      call inputData(iform,ni,mu,no,nons)

c     set array dimensions

      ni8=ni+8
      mu8=mu+8
      ngrid=ni*mu
      ngrid8    = ni8*mu8

      length1   = no*ngrid
      length2   = no*ngrid

      if     (iform.eq.1.or.iform.eq.3) then
         length3=( no*(no+1)/2+nons*(nons+1)/2 )*ngrid8
      elseif (iform.eq.0.or.iform.eq.2) then
         length3 = (no+nons)*ngrid8	 
      endif

c     ngrid8 is used below to allow excp to be used as a working array in fockDFT

c     OED
      if (imethod.eq.2 ) length3 =( no*(no+1)/2+nons*(nons+1)/2 )*ngrid8

c     HFS+DFT
      if (imethod.eq.3.or.imethod.eq.4) length3 = ngrid8	 

c     SCMC calculates exchange potentials (as in HF case) and needs extra space to
c     store the local exchange potential at the end of excp array
      if (imethod.eq.5) then
         length3=( no*(no+1)/2+nons*(nons+1)/2 + 1 )*ngrid8 
      endif

      length4   = nsuppl*ngrid
      length5   = nsctch*ngrid8     

      length6   = 2*ngrid8+4*ni*(mu+16)+2*maxgrids*(ni+mu)
      lexchrecl = ni*mu*8

c     dynamic allocation of memory acccording to the current case 

      size=lengthfp

c      length=8*length1
      call memAlloc (cw_orb_add,  length1, size, offset1)

c      length=8*length2
      call memAlloc (cw_coul_add, length2, size, offset2)
	
c      length=8*length3
      call memAlloc (cw_exch_add, length3, size, offset3)

c      length=8*length4
      call memAlloc (cw_suppl_add,length4, size, offset4)

c      length=8*length5
      call memAlloc (cw_sctch_add,length5, size, offset5)

c      length=8*length6

c     for 32-bit indexes allocation for cw_sor could be halved by
c     changing i8 into i4 
      call memAlloc (cw_sor_add,  length6, i8, offset6)

c     zeroise arrays
      call zeroArrays (cw_orb_add(offset1),cw_coul_add(offset2),
     &     cw_exch_add(offset3),cw_suppl_add(offset4),
     &     cw_sctch_add(offset5),cw_sor_add(offset6))

c     initialize arrays within common blocks, 
c     initialize arrays of variable length and check input data
c     prepare environment for Poisson's equation solving routines
c     prepare arrays determining ordering of mesh points

      call initArrays (cw_suppl_add(offset4),cw_sor_add(offset6))
c     print out data defining the case

      call printCase

c     initialize orbitals, Coulomb and exchange potentials

      call initOrbPot (cw_orb_add(offset1),cw_coul_add(offset2),
     &     cw_exch_add(offset3),cw_suppl_add(offset4),
     &     cw_sctch_add(offset5))



c     prepare SCF process: normalize/orthogonalize orbitals, calculate
c     Lagrange multipliers prepare data for boundary value evaluation

      call prepSCF (cw_sor_add(offset6),cw_orb_add(offset1),
     &     cw_coul_add(offset2),cw_exch_add(offset3),
     &     cw_suppl_add(offset4),cw_sctch_add(offset5))



c  perform scf
      call  doSCF (cw_sor_add(offset6),cw_orb_add(offset1),
     &     cw_coul_add(offset2),cw_exch_add(offset3),
     &     cw_suppl_add(offset4),cw_sctch_add(offset5))

c      call printResults (cw_orb,cw_coul,cw_exch,cw_suppl,cw_sctch)

      call printResults (cw_orb_add(offset1),
     &     cw_coul_add(offset2),cw_exch_add(offset3),
     &     cw_suppl_add(offset4),cw_sctch_add(offset5))

      call memDealloc (cw_orb_add,  length1, size, offset1)
      call memDealloc (cw_coul_add, length2, size, offset2)
      call memDealloc (cw_exch_add, length3, size, offset3)
      call memDealloc (cw_suppl_add,length4, size, offset4)
      call memDealloc (cw_sctch_add,length5, size, offset5)
      call memDealloc (cw_sor_add,  length6, i8, offset6)

c record the wall time needed for the program run

      datetime=dtstart
      write(*,'(//,1x,a7,80a1)') 'start: ',(str(i),i=1,24)
      call  getDateTime(datetime)
      write(*,'(1x,a7,80a1)') 'stop:  ',(str(i),i=1,24)
      call separator

      if (next.eq.1) then
         goto 100
      else
         stop 'x2dhf'
      endif

      end
      
      include 'blk-data.inc'
      
